@echo off

rem #  ------------------------------------------------------------
rem #  Company: Software- und Organisations-Service GmbH
rem #  Purpose: Startscript for JobScheduler Agent
rem #  ------------------------------------------------------------

SETLOCAL

rem ### SETTINGS ##############################################

rem # This variable has to point to the installation path of 
rem # the JobScheduler Agent.
rem # If this variable not defined then the parent directory 
rem # of this startscript is used.
  
rem set SCHEDULER_HOME=


rem # The default port of the JobScheduler Agent 

set /a SCHEDULER_AGENT_DEFAULT_HTTP_PORT=4445


rem # The http port of the JobScheduler Agent can be set here,
rem # as command line option -http-port (see usage) or as
rem # environment variable. Otherwise the above default port 
rem # is used.
rem # If only a port is specified then the JobScheduler Agent 
rem # listens to all available network interfaces.
rem # It is the same like 0.0.0.0:port.
rem # Use the form [ip address or hostname]:port for indicating 
rem # which network interfaces the JobScheduler Agent should 
rem # listen to.
rem # The command line option -http-port beats the environment 
rem # variable SCHEDULER_HTTP_PORT and the environment variable 
rem # SCHEDULER_HTTP_PORT beats the default port from 
rem # SCHEDULER_AGENT_DEFAULT_HTTP_PORT(=4445).
rem ### NOTE:
rem # If you start the JobScheduler Agent with the command line 
rem # option -http-port then you must enter -http-port for 
rem # stop, status, restart too (see usage). It's recommended 
rem # to set this environment variable instead. 

rem set SCHEDULER_HTTP_PORT=


rem # In addition to the http port a https port of the 
rem # JobScheduler Agent can be set here, as command line option 
rem # -https-port (see usage) or as environment variable.
rem # If only a port is specified then the JobScheduler Agent 
rem # listens to all available network interfaces.
rem # It is the same like 0.0.0.0:port.
rem # Use the form [ip address or hostname]:port for indicating 
rem # which network interfaces the JobScheduler Agent should 
rem # listen to.
rem # The command line option -https-port beats the environment 
rem # variable SCHEDULER_HTTPS_PORT.

rem set SCHEDULER_HTTPS_PORT=


rem # Set the directory where the JobScheduler Agent has the
rem # configuration, logs, etc. 
rem # The default is SCHEDULER_HOME\var_SCHEDULER_HTTP_PORT

rem set SCHEDULER_DATA=


rem # Set the directory where the JobScheduler Agent log file 
rem # is created. The default is SCHEDULER_DATA\logs

rem set SCHEDULER_LOG_DIR=


rem # Set the directory where the JobScheduler Agent pid file 
rem # is created. The default is SCHEDULER_LOG_DIR

rem set SCHEDULER_PID_FILE_DIR=


rem # The working directory of the JobScheduler Agent is  
rem # SCHEDULER_HOME. Here you can set a different working 
rem # directory (e.g. %USERPROFILE%).

rem set SCHEDULER_WORK_DIR=


rem # Set the location of a script which is called by the 
rem # JobScheduler Agent to kill a process and it's children.

rem set SCHEDULER_KILL_SCRIPT=


rem # Actually JAVA_HOME is already set. If you want to use 
rem # another Java environment then you can set it here. If  
rem # no JAVA_HOME is set then the Java from the path is used.

rem set JAVA_HOME=


rem # With Java 1.8 the initial memory allocation has changed, 
rem # for details see https://kb.sos-berlin.com/x/aIC9
rem # As a result on start-up of the JobScheduler Agent an 
rem # excessive amount of virtual memory is assigned by Java.  
rem # The environment variables JAVA_OPTIONS and
rem # SCHEDULER_JOB_JAVA_OPTIONS can be used to apply memory 
rem # settings such as '-Xms100m' (default for JAVA_OPTIONS) 
rem # where the JobScheduler Agent uses the JAVA_OPTIONS and 
rem # each job which is started by the Agent uses 
rem # SCHEDULER_JOB_JAVA_OPTIONS.

rem set JAVA_OPTIONS=
rem set SCHEDULER_JOB_JAVA_OPTIONS=

rem ###########################################################


rem ##########################
rem # internal global vars
set /a CREATE_PID_FILE_TRIAL=0
set /a RESTART_AGENT=0
set SCHEDULER_COMMAND=%~1
set SCHEDULER_STARTSCRIPT=%~nx0
set /a SCHEDULER_TIMEOUT=-1
set SCHEDULER_LOG_BRIDGE=log4j
set /a LOOP_COUNTER=0
set WEBSERVICES=
set SCHEDULER_PORT=


rem ##########################
rem # set and check home
if not defined SCHEDULER_HOME set SCHEDULER_HOME=%~dp0..
call :normalize_home "%SCHEDULER_HOME%\dummy"
if not exist "%SCHEDULER_HOME%\lib" (
  echo ...the directory "%SCHEDULER_HOME%\lib" is missing!
  echo did you move the startscript?
  echo please set the SCHEDULER_HOME variable.
  call :more_info
  exit /b 3
)


rem ##########################
rem # set http port and service name
set KILL_SCRIPT_OPTION=
set HTTPS_PORT_OPTION=
:read_command_line_options
shift
set OPTION_VALUE=%~2
set OPTION_VALUE=%OPTION_VALUE:-http-port=%
set OPTION_VALUE=%OPTION_VALUE:-https-port=%
set OPTION_VALUE=%OPTION_VALUE:-data-directory=%
set OPTION_VALUE=%OPTION_VALUE:-timeout=%
set OPTION_VALUE=%OPTION_VALUE:-kill-script=%
set OPTION_VALUE=%OPTION_VALUE:-java-options=%
if "%~1" == "" goto read_command_line_options_end
if "%~1" == "-timeout" (
  if not "%~2" == "" (
    set /a SCHEDULER_TIMEOUT=%~2
    if ERRORLEVEL 1 (
      echo ...invalid value in -timeout=%~2
      exit /b %ERRORLEVEL%
    )
  )
)
if "%~1" == "-http-port" (
  if "%OPTION_VALUE%" == "" goto read_command_line_options
  set SCHEDULER_HTTP_PORT=%~2
)
if "%~1" == "-https-port" (
  if "%OPTION_VALUE%" == "" goto read_command_line_options
  set SCHEDULER_HTTPS_PORT=%~2
)
if "%~1" == "-data-directory" (
  if "%OPTION_VALUE%" == "" goto read_command_line_options
  set SCHEDULER_DATA=%~2
)
if "%~1" == "-kill-script" (
  if "%OPTION_VALUE%" == "" goto read_command_line_options
  set SCHEDULER_KILL_SCRIPT=%~2
)
if "%~1" == "-java-options" (
  if "%OPTION_VALUE%" == "" goto read_command_line_options
  set JAVA_OPTIONS=%~2
)
if "%~1" == "-job-java-options" (
  if "%OPTION_VALUE%" == "" goto read_command_line_options
  set SCHEDULER_JOB_JAVA_OPTIONS=%~2
)
goto read_command_line_options
:read_command_line_options_end
call :add_ip_address %SCHEDULER_AGENT_DEFAULT_HTTP_PORT% %SCHEDULER_IP_ADDRESS%
if not defined SCHEDULER_HTTP_PORT set SCHEDULER_HTTP_PORT=%SCHEDULER_URL_SCHEMA%
call :extract_port %SCHEDULER_HTTP_PORT%
set SERVICE_NAME=sos_jobscheduler_agent_%SCHEDULER_PORT%
if defined SCHEDULER_KILL_SCRIPT set KILL_SCRIPT_OPTION=-kill-script="%SCHEDULER_KILL_SCRIPT%"
if defined SCHEDULER_HTTPS_PORT set HTTPS_PORT_OPTION=-https-port=%SCHEDULER_HTTPS_PORT%


rem ##########################
rem # set pid file
if not defined SCHEDULER_DATA set SCHEDULER_DATA=%SCHEDULER_HOME%\var_%SCHEDULER_PORT%
if not defined SCHEDULER_LOG_DIR set SCHEDULER_LOG_DIR=%SCHEDULER_DATA%\logs
rem TODO mkdir config/private, tmp und SCHEDULER_DATA
if not exist "%SCHEDULER_LOG_DIR%" mkdir "%SCHEDULER_LOG_DIR%"
if ERRORLEVEL 1 (
  echo ...could not create "%SCHEDULER_LOG_DIR%"
  echo ...please check the permissions
  call :more_info
  exit /b %ERRORLEVEL%
)
if not defined SCHEDULER_PID_FILE_DIR set SCHEDULER_PID_FILE_DIR=%SCHEDULER_LOG_DIR%
if not exist "%SCHEDULER_PID_FILE_DIR%" mkdir "%SCHEDULER_PID_FILE_DIR%"
if ERRORLEVEL 1 (
  echo ...could not create "%SCHEDULER_PID_FILE_DIR%"
  echo ...please check the permissions
  call :more_info
  exit /b %ERRORLEVEL%
)
if not exist "%SCHEDULER_DATA%\tmp" mkdir "%SCHEDULER_DATA%\tmp"
if ERRORLEVEL 1 (
  echo ...could not create "%SCHEDULER_DATA%\tmp"
  echo ...please check the permissions
  call :more_info
  exit /b %ERRORLEVEL%
)
set SCHEDULER_PID_FILE=%SCHEDULER_PID_FILE_DIR%\jobscheduler_agent_%SCHEDULER_PORT%.pid


rem ##########################
rem # set log file basename
call :set_short_filename SCHEDULER_LOG_DIR "%SCHEDULER_LOG_DIR%"
set SCHEDULER_LOGFILE=%SCHEDULER_LOG_DIR%\jobscheduler_agent_%SCHEDULER_PORT%


rem ##########################
rem # set working directory
if not defined SCHEDULER_WORK_DIR set SCHEDULER_WORK_DIR=%SCHEDULER_HOME%


rem ##########################
rem # set kill script location etc.
if not defined KILL_TASK_LOG_FILE set KILL_TASK_LOG_FILE=%SCHEDULER_LOG_DIR%\agent_kill_task_%SCHEDULER_PORT%.log
set SCHEDULER_CRASH_FILE=%SCHEDULER_DATA%\tmp\kill_tasks_after_crash.cmd


rem ##########################
rem # set java options
if not defined JAVA_OPTIONS set JAVA_OPTIONS=-Xms100m
if defined SCHEDULER_JOB_JAVA_OPTIONS set SCHEDULER_JOB_JAVA_OPTIONS=-DLOGFILE=%SCHEDULER_LOGFILE% %SCHEDULER_JOB_JAVA_OPTIONS%
if not defined SCHEDULER_JOB_JAVA_OPTIONS set SCHEDULER_JOB_JAVA_OPTIONS=-DLOGFILE=%SCHEDULER_LOGFILE%



rem ##########################
rem # set and check java
set JAVAWBIN=javaw.exe
set JAVABIN=java.exe
if exist "%JAVA_HOME%\bin\javaw.exe" set JAVAWBIN=%JAVA_HOME%\bin\javaw.exe
if exist "%JAVA_HOME%\bin\java.exe" set JAVABIN=%JAVA_HOME%\bin\java.exe



rem ##########################
rem # ... and action
:action
if "%SCHEDULER_COMMAND%"=="start"   goto do_start
if "%SCHEDULER_COMMAND%"=="stop"    goto do_stop
if "%SCHEDULER_COMMAND%"=="abort"   goto do_abort
if "%SCHEDULER_COMMAND%"=="restart" goto do_restart
if "%SCHEDULER_COMMAND%"=="status"  goto do_status
if "%SCHEDULER_COMMAND%"=="debug"   goto do_debug
if "%SCHEDULER_COMMAND%"=="kill"    goto do_kill
if "%SCHEDULER_COMMAND%"=="install-service" goto do_install_service
if "%SCHEDULER_COMMAND%"=="install_service" goto do_install_service
if "%SCHEDULER_COMMAND%"=="install" goto do_install_service
if "%SCHEDULER_COMMAND%"=="remove-service"  goto do_remove_service
if "%SCHEDULER_COMMAND%"=="remove_service"  goto do_remove_service
if "%SCHEDULER_COMMAND%"=="remove"  goto do_remove_service
if "%SCHEDULER_COMMAND%"=="start-service"   goto do_start_service
if "%SCHEDULER_COMMAND%"=="start_service"   goto do_start_service
rem is called only at the end of the installer
if "%SCHEDULER_COMMAND%"=="setup_start_service" goto do_setup_start_service
goto usage


:usage
echo Usage: %SCHEDULER_STARTSCRIPT% command [options] [web services]
echo   command:
echo     start           [options]
echo     stop            [options]
echo     abort           [options]
echo     restart         [options]
echo     status          [options] [web services]
echo     debug           [options]
echo     kill            [options]
echo     install-service [options]
echo     remove-service  [options]
echo     start-service   [options]
echo   options:
echo     -http-port=^<[hostname or ip address:]number^>               ^| default: %SCHEDULER_HTTP_PORT%
echo     -https-port=^<[hostname or ip address:]number^>              ^| default: %SCHEDULER_HTTPS_PORT%
echo     -data-directory=^<location of data directory^>               ^| default: %SCHEDULER_DATA%
echo     -timeout=^<number^>                                          ^| in seconds; only for stop and restart
echo     -ip-address=^<hostname or ip address^>                       ^| to listen to a specific host name or ip address
echo     -kill-script=^<location of kill script^>                     ^| only for start
echo     -java-options=^<java options^>                               ^| default: %JAVA_OPTIONS%; see also https://kb.sos-berlin.com/x/aIC9
echo     -job-java-options=^<java options^>                           ^| see also https://kb.sos-berlin.com/x/aIC9
echo   web services:
echo     overview                                                   ^| default
echo     task                                                       ^| running JobScheduler Agent tasks
echo     tunnel                                                     ^| tunnels summary
echo     tunnel/                                                    ^| list of tunnels
echo     command                                                    ^| running command summary
echo     command/                                                   ^| list of running commands ^(watching file_order_source^)
call :more_info
exit /b 0


:more_info
echo see also https://kb.sos-berlin.com/x/kQiX for more information.
goto final


:do_start
call :running
if %SCHEDULER_IS_RUNNING% equ 1 (
  call :message "...JobScheduler Agent(%SCHEDULER_PORT%) already running"
  exit /b 0 
)
call :remove_crash_file
set SCHEDULER_LIBDIR=%SCHEDULER_HOME:\=/%/lib
set SCHEDULER_CLASSPATH=%SCHEDULER_LIBDIR%/log/%SCHEDULER_LOG_BRIDGE%/*;%SCHEDULER_LIBDIR%/jdbc/*;%SCHEDULER_LIBDIR%/3rd-party/*;%SCHEDULER_LIBDIR%/sos/*;%SCHEDULER_LIBDIR%/
  
start "JobSchedulerAgentW" /B "%COMSPEC%" /C ""%SCHEDULER_HOME%\bin\jobscheduler_agent_watchdog.cmd"" -%SCHEDULER_PORT%
if ERRORLEVEL 1 (
  rem TODO ERRORLEVEL is always 0
  exit /b %ERRORLEVEL%
) else (
  goto create_pid_file
)
goto finally


:do_debug
call :running
if %SCHEDULER_IS_RUNNING% equ 1 (
  call :message "...JobScheduler Agent(%SCHEDULER_PORT%) already running"
  exit /b 0 
)
call :remove_crash_file
set CURDIR=%CD%
cd /D "%SCHEDULER_WORK_DIR%"
set SCHEDULER_LIBDIR=%SCHEDULER_HOME:\=/%/lib
set SCHEDULER_CLASSPATH=%SCHEDULER_LIBDIR%/log/%SCHEDULER_LOG_BRIDGE%/*;%SCHEDULER_LIBDIR%/jdbc/*;%SCHEDULER_LIBDIR%/3rd-party/*;%SCHEDULER_LIBDIR%/sos/*;%SCHEDULER_LIBDIR%/
"%JAVABIN%" %JAVA_OPTIONS% -classpath "%SCHEDULER_CLASSPATH%" com.sos.scheduler.engine.agent.main.AgentMain -http-port=%SCHEDULER_HTTP_PORT% %HTTPS_PORT_OPTION% -data-directory="%SCHEDULER_DATA%" -log-directory="%SCHEDULER_LOG_DIR%" %KILL_SCRIPT_OPTION%
if ERRORLEVEL 1 (
  cd /D "%CURDIR%"
  exit /b %ERRORLEVEL%
) else (
  cd /D "%CURDIR%"
  goto create_pid_file
)
goto finally


:do_stop
call :running
if %SCHEDULER_IS_RUNNING% equ 0 (
  echo ...not started!
  goto finally
)
set /a RESTART_AGENT=%RESTART_AGENT%+%SCHEDULER_RUNNING_AS_SERVICE%
call :message "...try to terminate the JobScheduler Agent(%SCHEDULER_PORT%)"
if %SCHEDULER_TIMEOUT% lss 0 (
  call :command "{$TYPE: Terminate, sigtermProcesses: false}"
) else (
  call :command "{$TYPE: Terminate, sigtermProcesses: false, sigkillProcessesAfter: %SCHEDULER_TIMEOUT%}"
)
if ERRORLEVEL 1 (
  call :message "...could not terminate the JobScheduler Agent(%SCHEDULER_PORT%)"
  exit /b 3
) else (
  call :sleep 2
  call :wait_for_termination
  echo ...terminated!
  call :remove_pid_file
)
if %RESTART_AGENT% equ 3 goto do_start_service
if %RESTART_AGENT% equ 2 goto do_start
goto finally


:do_abort
call :running
if %SCHEDULER_IS_RUNNING% equ 0 (
  echo ...not started!
  goto finally
)
call :message "...try to abort the JobScheduler Agent(%SCHEDULER_PORT%)"
call :command "{$TYPE: AbortImmediately}"
call :sleep 2
call :running
if %SCHEDULER_IS_RUNNING% equ 1 (
  call :message "...could not abort the JobScheduler Agent(%SCHEDULER_PORT%)"
  exit /b 3
) else (
  echo ...aborted!
  call :remove_pid_file
)
goto finally


:do_kill
call :running
if %SCHEDULER_IS_RUNNING% equ 0 (
  echo ...not started!
  goto finally
)
if x%SCHEDULER_PID%==x (
  echo ...the pid of the JobScheduler Agent^(%SCHEDULER_PORT%^) process couldn't determine!
  exit /b 3
)
call :message "...try to kill the JobScheduler Agent(%SCHEDULER_PORT%)"
taskkill /F /PID %SCHEDULER_PID% > nul
if ERRORLEVEL 1 (
  call :message "...could not kill the JobScheduler Agent(%SCHEDULER_PORT%)"
  exit /b 3
) else (
  echo ...killed!
  call :remove_pid_file
) 
goto finally


:do_restart
set /a RESTART_AGENT=2
goto do_stop


:do_status
set SCHEDULER_STARTSCRIPT_OPTS=%*
call :replace_equal_sign SCHEDULER_STARTSCRIPT_OPTS "-"
call :extract_web_services %SCHEDULER_STARTSCRIPT_OPTS%
if not defined WEBSERVICES set WEBSERVICES="/overview"

call :command %WEBSERVICES%
if ERRORLEVEL 1 (
  echo ...JobScheduler Agent^(%SCHEDULER_PORT%^) not started!
  exit /b 3
)
goto finally


:do_install_service
if not exist "%SCHEDULER_HOME%\service\install_agent_windows_service.cmd" (
  echo ...service install script "..\service\install_agent_windows_service.cmd" is missing.
  exit /b 2
)
call "%SCHEDULER_HOME%\service\install_agent_windows_service.cmd" "%SCHEDULER_HTTP_PORT%" "%SCHEDULER_DATA%" "%SCHEDULER_LOG_DIR%" "%HTTPS_PORT_OPTION%" "" "" "%SCHEDULER_HOME%"
goto finally


:do_remove_service
if not exist "%SCHEDULER_HOME%\service\uninstall_agent_windows_service.cmd" (
  echo ...service remove script "..\service\uninstall_agent_windows_service.cmd" is missing.
  exit /b 2
)
call "%SCHEDULER_HOME%\service\uninstall_agent_windows_service.cmd" "%SCHEDULER_PORT%" "%SCHEDULER_DATA%" "%SCHEDULER_LOG_DIR%" "%SCHEDULER_HOME%"
goto finally


:do_start_service
call :running
if %SCHEDULER_IS_RUNNING% equ 1 (
  call :message "...JobScheduler Agent(%SCHEDULER_PORT%) already running"
  exit /b 0 
)
if %RESTART_AGENT% equ 0 sc.exe query %SERVICE_NAME%
if ERRORLEVEL 1 goto finally
"%SCHEDULER_HOME%\service\%SERVICE_NAME%.exe" //ES/%SERVICE_NAME%
if ERRORLEVEL 1 (
  echo Failed to start JobScheduler Agent^(%SCHEDULER_PORT%^) service. Refer to log in "%SCHEDULER_LOG_DIR%"
  exit /b 2
)
sc.exe query %SERVICE_NAME%
echo ...JobScheduler Agent^(%SCHEDULER_PORT%^) service has been started.
goto finally


:do_setup_start_service
call :do_start_service
if ERRORLEVEL 1 exit /b 0
echo.
echo ...check connection in 10 seconds
call :sleep 10
echo.
call :do_status /overview
exit /b 0


:command
set SCHEDULER_LIBDIR=%SCHEDULER_HOME:\=/%/lib
set SCHEDULER_CLASSPATH=%SCHEDULER_LIBDIR%/log/%SCHEDULER_LOG_BRIDGE%/*;%SCHEDULER_LIBDIR%/3rd-party/*;%SCHEDULER_LIBDIR%/sos/*;%SCHEDULER_LIBDIR%/
call :add_ip_address %SCHEDULER_HTTP_PORT% 127.0.0.1
"%JAVABIN%" -DLOGFILE="%SCHEDULER_LOGFILE%" %JAVA_OPTIONS% -classpath "%SCHEDULER_CLASSPATH%" com.sos.scheduler.engine.agent.client.main.AgentClientMain http://%SCHEDULER_URL_SCHEMA% %*
exit /b %ERRORLEVEL%
goto final


:running
set /a SCHEDULER_IS_RUNNING=0
set SCHEDULER_PID=
set SCHEDULER_PID_FROMWMIC=
if not exist "%SCHEDULER_PID_FILE%" goto running2
for /f "usebackq tokens=1" %%i in (`type "%SCHEDULER_PID_FILE%"`) do set SCHEDULER_PID=%%i
if x%SCHEDULER_PID%==x goto running2
rem check if process exist with pid from pidfile
for /f "usebackq tokens=2 delims==" %%i in (`wmic process where "processid=%SCHEDULER_PID%" get processid /format:value 2^>nul`) do set SCHEDULER_PID_FROMWMIC=%%i
if x%SCHEDULER_PID_FROMWMIC% == x%SCHEDULER_PID% set /a SCHEDULER_IS_RUNNING=1
goto running3
:running2
call :command "/overview" >nul 2>nul
if %ERRORLEVEL% equ 0 set /a SCHEDULER_IS_RUNNING=1
:running3
call :running_as_service
if %SCHEDULER_IS_RUNNING% equ 0 goto remove_pid_file
goto final


:running_as_service
set /a SCHEDULER_RUNNING_AS_SERVICE=0
for /F "usebackq tokens=3 delims=: " %%i in (`sc.exe query %SERVICE_NAME% ^| findstr /I "STATE"`) do (
  if /I x%%i==xRUNNING set /a SCHEDULER_RUNNING_AS_SERVICE=1
)
goto final


:message
echo|set /p="%~1 "
if %SCHEDULER_RUNNING_AS_SERVICE% equ 1 echo|set /p="as service "
if x%SCHEDULER_PID%==x (
  echo !
) else (
  echo with pid=%SCHEDULER_PID%!
)
goto final


:wait_for_termination
call :sleep 2
call :running
if %SCHEDULER_IS_RUNNING% equ 0 goto final
set /a LOOP_COUNTER=%LOOP_COUNTER%+2
if %LOOP_COUNTER% equ 2 echo|set /p=...waiting for termination
echo|set /p=..
goto wait_for_termination


:normalize_home
set SCHEDULER_HOME=%~dp1
set SCHEDULER_HOME=%SCHEDULER_HOME:~0,-1%
goto final


:sleep
"%SCHEDULER_HOME%\bin\sleep.exe" %1
exit /B 0
goto final


:create_pid_file
set /a CREATE_PID_FILE_TRIAL=%CREATE_PID_FILE_TRIAL%+1
set SCHEDULER_PID_FROMWMIC=
if %CREATE_PID_FILE_TRIAL% geq 3 (
  echo ...the pid of the JobScheduler Agent^(%SCHEDULER_PORT%^) process couldn't determine!
  exit /b 3
)
for /f "usebackq tokens=2 delims==" %%i in (`wmic process where "commandline like '%%com.sos.scheduler.engine.agent.main.AgentMain%%-http-port=%SCHEDULER_HTTP_PORT%%%' and caption like '%%java%%'" get processid /format:value 2^>nul`) do set SCHEDULER_PID_FROMWMIC=%%i
if x%SCHEDULER_PID_FROMWMIC% == x (
  call :sleep 2
  goto create_pid_file
) else (
  echo %SCHEDULER_PID_FROMWMIC% > "%SCHEDULER_PID_FILE%"
)
if exist "%SCHEDULER_PID_FILE%" (
  echo ...JobScheduler Agent^(%SCHEDULER_PORT%^) started with pid=%SCHEDULER_PID_FROMWMIC%! 
  echo ...see log file "%SCHEDULER_LOGFILE%.log"
) else (
  echo ...pid file "%SCHEDULER_PID_FILE%" could not created!
  echo ...please check the permissions in "%SCHEDULER_PID_FILE_DIR%".
  call :more_info
  rem taskkill /F /PID %SCHEDULER_PID_FROMWMIC% > nul
  exit /b 3
)
goto finally


:remove_pid_file
if exist "%SCHEDULER_PID_FILE%" del "%SCHEDULER_PID_FILE%"  
goto final


:remove_crash_file
if exist "%SCHEDULER_CRASH_FILE%" del "%SCHEDULER_CRASH_FILE%"  
goto final


:replace_equal_sign
SETLOCAL ENABLEDELAYEDEXPANSION
set "work=!%1!"
set /a i=0
:replace_loop
if "!work:~%i%,1!"=="" (
  ENDLOCAL&set "%1=%work%"
  goto final
)
if not "!work:~%i%,1!"=="=" (
  set /a i+=1
  goto replace_loop
)
set head=!work:~0,%i%!
set /a i+=1
set "work=!head!%~2!work:~%i%!"
goto replace_loop


:extract_web_services
set "work=%~1"
if "%work%"=="%SCHEDULER_COMMAND%" shift
set "work=%~1"
if "%work:~0,1%"=="-" (
  shift
  goto extract_web_services
)
set "work=%~1"
if "%work%"=="" goto final
set "WEBSERVICE=%~1"
if "%WEBSERVICE:~0,1%"=="{" goto extract_web_service
if not "%WEBSERVICE:~0,1%"=="/" set "WEBSERVICE=/%WEBSERVICE%"
:extract_web_service
set WEBSERVICES=%WEBSERVICES% "%WEBSERVICE%"
shift
goto extract_web_services
goto final

:add_ip_address
set NEW_IP_ADDRESS=%~2
if defined NEW_IP_ADDRESS set NEW_IP_ADDRESS=%NEW_IP_ADDRESS%:
for /F "tokens=1,2 delims=:" %%i in ("%~1") do (
  if x%%j==x (
    call :set_url_schema %NEW_IP_ADDRESS%%%i
    goto final
  )
  if x%%i==x0.0.0.0 (
    call :set_url_schema %NEW_IP_ADDRESS%%%j
    goto final
  )
  call :set_url_schema %~1
  goto final
)

:extract_port
for /F "tokens=1,2 delims=:" %%i in ("%~1") do (
  if x%%j==x (
    call :set_port %%i
    goto final
  )
  call :set_port %%j
  goto final
)

:set_url_schema
set SCHEDULER_URL_SCHEMA=%~1
goto final

:set_port
set SCHEDULER_PORT=%~1
goto final

:set_short_filename
set "%1=%~s2"
goto final

:finally
exit /b %ERRORLEVEL%

ENDLOCAL
:final